UNIT frmMain;
{+F
---------------------------------------------------------------------------
  File        - FRMMAIN.PAS
  Purpose     - Demonstrate a menu item color square
  Author      - Keith S. Brown
  Copyright    NONE. Released to public domain by KSS.
                http://www.ghg.net/chairouchu
  Revised     - 2000.10.29(KSB) Created original.
---------------------------------------------------------------------------}
INTERFACE
USES
  Windows,       Messages,      SysUtils,      Classes,       Graphics,
  Controls,      Forms,         Dialogs,       Menus;

TYPE
  TdlgMain    = CLASS(TForm)
    mnuMain   : TMainMenu;
    mniExit   : TMenuItem;
    mniFile   : TMenuItem;
    mniColors : TMenuItem;
    mniOption1: TMenuItem;
    mniOption2: TMenuItem;
    mniOption3: TMenuItem;
    mniOption4: TMenuItem;
    dlgColor  : TColorDialog;
    PROCEDURE   FormCreate(aoSender: TObject);
    PROCEDURE   MenuItemAdvancedDraw(aoSender: TObject; aoCanvas: TCanvas; arRect: TRect; asetState: TOwnerDrawState);
    PROCEDURE   MenuItemClick(aoSender: TObject);
    PROCEDURE   MenuItemMeasureItem(aoSender: TObject; aoCanvas: TCanvas; VAR aiWidth, aiHeight: INTEGER);
    PROCEDURE   mniExitClick(aoSender: TObject);
  PRIVATE
  PUBLIC
  END {CLASS};

VAR
  dlgMain     : TdlgMain;
              {=================================================}
IMPLEMENTATION
{$R *.DFM}

{}PROCEDURE   TdlgMain.FormCreate(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Enable color squares:
                1. parent menu must have OwnerDraw = TRUE
                2. store the color values in the tag property for the menu
                   item
---------------------------------------------------------------------------}
  BEGIN
    mnuMain.OwnerDraw := TRUE;
    mniOption1.Tag := clRed;
    mniOption2.Tag := clBlue;
    mniOption3.Tag := clGreen;
    mniOption4.Tag := clBtnFace;
{}END {FormCreate};


              {Allow access to the protected TMenuItem.DoDrawText method
               by defining a cheater class (Classes defined in the same
               unit have access to each other's protected methods and
               properties...)}
TYPE
  TmniCheater = CLASS(TMenuItem);

{}PROCEDURE   TdlgMain.MenuItemAdvancedDraw(aoSender: TObject; aoCanvas: TCanvas; arRect: TRect; asetState: TOwnerDrawState);
{+M
---------------------------------------------------------------------------
  Purpose     - Modify the menu item so that it draws a color square on the
                left.
---------------------------------------------------------------------------}
  VAR
    liWidth   : INTEGER;
    lrRect    : TRect;
  BEGIN
    IF aoSender IS TMenuItem THEN BEGIN
              {Define the top and bottom of the basic color square}
      lrRect.Top := arRect.Top;
      lrRect.Bottom := arRect.Bottom;
              {The width is the height of the color square ([0,0] pixel
               is in the upper left). Store the width, we'll need it later.}
      liWidth := lrRect.Bottom - lrRect.Top;
      lrRect.Left := 0;
      lrRect.Right := liWidth;
              {draw a border around the color square}
      InflateRect(lrRect,-2,-2);
      aoCanvas.Brush.Color := clBlack;
      aoCanvas.FillRect(lrRect);
              {draw the color square}
      InflateRect(lrRect,-1,-1);
      aoCanvas.Brush.Color := TMenuItem(aoSender).Tag;
      aoCanvas.FillRect(lrRect);
              {expand the menu text rectangle so the text won't be truncated}
      arRect.Left := arRect.Left + liWidth;
      arRect.Right:= arRect.Right+ liWidth;
              {draw the rest of the menu item}
      WITH TmniCheater(aoSender) DO BEGIN
        DoDrawText(aoCanvas, Caption, arRect, odSelected IN asetState, DT_LEFT);
      END {WITH};
    END {IF};
{}END {MenuItemAdvanceDraw};




{}PROCEDURE   TdlgMain.MenuItemClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Execute the color dialog to select new colors for the
                selected color menu item.
---------------------------------------------------------------------------}
  BEGIN
    IF aoSender IS TMenuItem THEN BEGIN
      dlgColor.Color := TMenuItem(aoSender).Tag;
      IF dlgColor.Execute THEN BEGIN
        TMenuItem(aoSender).Tag := dlgColor.Color;
      END {IF};
    END {IF};
{}END {MenuItemClick};




{}PROCEDURE   TdlgMain.MenuItemMeasureItem(aoSender: TObject; aoCanvas: TCanvas; VAR aiWidth, aiHeight: INTEGER);
{+M
---------------------------------------------------------------------------
  Purpose     - Resize the menu item such that both the color square and the
                original text will be displayed.
---------------------------------------------------------------------------}
  BEGIN
    aiWidth := aiWidth + aiHeight;
{}END {MenuItemMeasureItem};




{}PROCEDURE   TdlgMain.mniExitClick(aoSender: TObject);
{+M
---------------------------------------------------------------------------
  Purpose     - Exit the application by closing the window.
---------------------------------------------------------------------------}
  BEGIN
    Close;
{}END {mniExitClick};




END {frmMain}.
